@@ -1,5 +1,7 @@
 """A module for the first-class manipulation of physical quantities.
 
+It is named ``pyunitx`` as a reference to the ``siunitx`` LaTeX package.
+
 Most conversion factors come from
 `this document from NIST <https://physics.nist.gov/cuu/pdf/sp811.pdf>`_.
 Most values of constants come from
@@ -12,9 +14,10 @@ Most values of constants come from
 # available.
 # noinspection PyUnresolvedReferences
 from . import (
-    _api,
     angle,
+    charge,
     constants,
+    current,
     derived,
     energy,
     force,
@@ -23,8 +26,18 @@ from . import (
     mole,
     power,
     pressure,
+    resistance,
     temperature,
     time,
-    velocity,
     volume,
+    voltage,
+)
+
+from ._api import (
+    make_dimension,
+    make_unit,
+    make_compound_dimension,
+    make_compound_unit,
+    SIUNITX_OLD,
+    SIUNITX_NEW,
 )
