@@ -1,7 +1,9 @@
+import re
 import textwrap
+import warnings
 from decimal import Decimal
 from numbers import Number
-from typing import Union, Tuple, Sequence, Type, Dict, Iterator
+from typing import Union, Tuple, Sequence, Type, Dict, Iterator, Optional
 
 import sigfig
 
@@ -12,6 +14,8 @@ __all__ = [
     "make_unit",
     "make_compound_dimension",
     "make_compound_unit",
+    "SIUNITX_NEW",
+    "SIUNITX_OLD",
 ]
 UnitOperand = Union['UnitInterface', Number]
 Scale = Union[Decimal, float, str, Tuple[int, Sequence[int], int]]
@@ -20,11 +24,16 @@ Pair = Tuple[Unitlike, int]
 Pairs = Tuple[Pair, ...]
 Exponents = Union[Pairs, Dict[Unitlike, int]]
 
+SIUNITX_NEW = 3
+SIUNITX_OLD = 2
+
+_EXTANT_UNITS = {}
+
 
 def make_dimension(name: str) -> 'DimensionBase':
     """Dimensions are the basic measurable quantities about the world, like length and time.
 
-    For complex dimensions, see make_compound_dimension.
+    For complex dimensions, see :func:`make_compound_dimension`.
 
     :param name: The name of this dimension.
     :return: An object that captures information about the dimension and the
@@ -34,14 +43,17 @@ def make_dimension(name: str) -> 'DimensionBase':
     return dimension
 
 
-def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, doc="") -> Type[
-    'UnitInterface']:
+def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev: str, doc="") \
+        -> Type['UnitInterface']:
     """A unit is a particular convention for measuring a dimension.
 
     The resulting class represents the abstract concept of the unit, say meters.
     Instances of the class associate that unit with a number, defining in this
     example a particular length like 8 meters. Instances of a unit class are
-    called "measurements" within this documentation.
+    called "measurements" within this documentation. Instances are immutable;
+    any interactions with the public API will not change the object, anything
+    that appears like it (for instance rounding with ``.sig_figs``) instead
+    constructs and returns a copy.
 
     :param name: The name of the new unit, like "meters". Should be in its
         plural form.
@@ -50,7 +62,8 @@ def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, do
         current unit. For instance, one kilometer has a scale of 1000 when the
         base unit is meters.
     :param abbrev: The canonical abbreviation for this unit, like "lb" for
-        pounds or "g" for grams. Used for output.
+        pounds or "g" for grams. Officially called the unit's symbol. Used for
+        output.
     :param doc: Documentation for this unit, optional.
     :return: A class representing the unit. Instances of this class are
         measurements using the unit.
@@ -107,22 +120,27 @@ def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, do
             return False
         return self.value * self.scale == other.value * other.scale
 
-    def equivalent(self, other: UnitInterface, within=0) -> bool:
+    def equivalent(self, other: UnitInterface, figs=5) -> bool:
         """Check if this measurement represents the same quantity as another,
         within a certain precision.
 
-        You can think of it as converting both measurements to their shared base
-        unit, then checking whether they are within the specified difference of
-        each other.
+        This effectively converts both units to the base unit to get them on
+        equal footing before rounding and checking equality.
 
         :param other: The other measurement.
-        :param within: An absolute tolerance on the approximation, expressed in
-            the base unit.
+        :param figs: How many significant figures to round to before comparison.
         :return: Whether the two measurements are approximately equal.
         """
         if not other.is_dimension(self.dimension):
             raise ImplicitConversionError(type(other), type(self))
-        return abs(self.value * self.scale - other.value * other.scale) <= within
+        with warnings.catch_warnings():
+            # sigfig.round raises warnings if the quantity being rounded has fewer significant
+            # figures than are given, but for the purposes of this comparison, I don't care
+            # so ignore them all
+            warnings.simplefilter("ignore")
+            a = sigfig.round(self.value * self.scale, sigfigs=figs)
+            b = sigfig.round(other.value * other.scale, sigfigs=figs)
+        return a == b
 
     def multiply(self, other: Union[UnitInterface, int, float, Decimal]) -> UnitInterface:
         """Multiply two measurements. Produces a new compound unit for the result.
@@ -215,6 +233,18 @@ def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, do
             name.
         """
         if key.startswith("to_"):
+            if not hasattr(self.dimension, key):
+                composition = Compound.from_string(key[3:])
+                base_scale = 1
+                for u, e in composition.to_pairs():
+                    base_scale *= u.scale ** e
+                make_compound_unit(
+                    name=key[3:],
+                    scale=base_scale,
+                    exponents=composition.to_pairs(),
+                )
+                # It is automatically registered on the dimension on creation,
+                # no need to explicitly use it
             return lambda: getattr(self.dimension, key)(self)
         raise AttributeError()
 
@@ -230,8 +260,38 @@ def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, do
         rounded = sigfig.round(self.value, sigfigs=figs)
         return type(self)(rounded)
 
-    # noinspection PyTypeChecker
+    def to_latex(self, siunitx_major_version=SIUNITX_NEW):
+        r"""Output this unit value as it would be used in LaTeX with siunitx.
+
+        Maybe script your calculations, then write the output to a .tex file
+        and include it in your main document.
+
+        :param siunitx_major_version: siunitx changed the name of its macro
+            that outputs a number with a unit in version 3.0. Prior, it was
+            ``\SI{number}{units}``, and after it was ``\qty{number}{units}``.
+            Defaults to the most recent version. If you're using an older one,
+            for instance if your TeXLive distribution is a few years old, pass
+            in :data:`pyunitx.SIUNITX_OLD`, an alias for 2. I doubt anyone is
+            using version 1.
+        :return: A string formatted for direct inclusion in LaTeX.
+        """
+        if siunitx_major_version >= 3:
+            fmt = r"\qty{{{value}}}{{{unit}}}"
+        else:
+            fmt = r"\SI{{{value}}}{{{unit}}}"
+        if re.match(r"^\w+$", self.abbreviation):
+            # This unit has an elementary name, like J for joules
+            return fmt.format(value=self.value, unit=self.abbreviation)
+        units = []
+        for u, e in self.composition.to_pairs():
+            if e == 1:
+                units.append(u.abbreviation)
+            else:
+                units.append(f"{u.abbreviation}^{{{e}}}")
+        return fmt.format(value=self.value, unit=".".join(units))
+
     # @formatter:off
+    # noinspection PyTypeChecker
     unit: Type[UnitInterface] = type(name, (object,), {
         "__new__": new,
         "__add__": add,
@@ -255,6 +315,7 @@ def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, do
         "equivalent_to": equivalent,
         "sig_figs": sig_figs,
         "abbreviation": abbrev,
+        "to_latex": to_latex,
     })
     # @formatter:on
     unit.composition = Compound(((unit, 1),))
@@ -264,6 +325,7 @@ def make_unit(*, name: str, dimension: 'DimensionBase', scale: Scale, abbrev, do
         return unit(self.value * self.scale / unit.scale)
 
     setattr(dimension, "to_" + name.replace(" ", "_"), converter)
+    _EXTANT_UNITS[unit.__name__] = unit
     return unit
 
 
@@ -292,7 +354,6 @@ def make_compound_unit(*, scale: Scale, exponents: Exponents, name: str = None,
                        doc=""):
     """A unit composed of other units.
 
-    :param abbrev:
     :param scale: How many of the base unit one of this unit is equivalent to.
         Note that the scale factors of the constituent units are not taken into
         account. For instance, even though the conversion factor from meters per
@@ -305,15 +366,22 @@ def make_compound_unit(*, scale: Scale, exponents: Exponents, name: str = None,
     :param name: A name to give this unit, like joules. If no particular
         physical name exists, leave it unfilled and a name will be automatically
         constructed using the names of its component units and their exponents.
+    :param abbrev: A canonical abbreviation for this unit, like J for joules.
+        Officially called the unit's symbol. If no particular one exists, leave
+        it unfilled and it will be automatically be constructed using the
+        symbols of its base units.
     :param doc: Documentation for this unit, to show in tools like Sphinx.
     :return: A class representing this unit. Instances of the class are
         measurements with a magnitude.
     """
     if isinstance(exponents, dict):
         exponents = tuple(exponents.items())
+    composition = Compound(exponents)
+    existing = _access_unit_cache(composition, scale)
+    if existing:
+        return existing
     if name is None:
         name = str(Multiset(exponents))
-    composition = Compound(exponents)
     dims_extracted = tuple((unit.dimension, exp) for unit, exp in composition.to_pairs())
     dims = _sort(_dedupe(dims_extracted))
     dimension = make_compound_dimension(dims)
@@ -379,6 +447,15 @@ def _sort(d: Union[Exponents, Iterator[Pair]]) -> Pairs:
     return tuple(sorted(pairs, key=lambda p: (-p[1], p[0].__name__)))
 
 
+def _access_unit_cache(c: 'Compound', scale) -> Optional['UnitInterface']:
+    remap = {(_sort(_dedupe(unit.composition.to_pairs())), unit.scale): unit
+             for name, unit in _EXTANT_UNITS.items()}
+    key = (_sort(_dedupe(c.to_pairs())), scale)
+    if key in remap:
+        return remap[key]
+    return None
+
+
 class DimensionBase:
     __INSTANCES = {}
 
@@ -399,10 +476,10 @@ class DimensionBase:
             key = _decompose_all(exponents)
         if key not in cls.__INSTANCES:
             cls.__INSTANCES[key] = super(DimensionBase, cls).__new__(cls)
-        this = cls.__INSTANCES[key]
-        this.__name__ = name
-        this.composition = Compound(exponents or ((this, 1),))
-        return this
+            this = cls.__INSTANCES[key]
+            this.__name__ = name
+            this.composition = Compound(exponents or ((this, 1),))
+        return cls.__INSTANCES[key]
 
     def __hash__(self):
         return hash(self.__name__)
@@ -511,7 +588,7 @@ class Compound:
                     f"The exponentiation has caused a non-integer exponent on {t}, "
                     f"which is not allowed."
                 )
-            pairs.append((t, new_exponent))
+            pairs.append((t, int(new_exponent)))
         return Compound(tuple(pairs))
 
     def __verify_no_dimension_mismatch(self, extra: Multiset):
@@ -536,6 +613,32 @@ class Compound:
                 chunks.append(f"{unit.abbreviation}^{exponent}")
         return " ".join(chunks)
 
+    @classmethod
+    def from_string(cls, spec: str) -> 'Compound':
+        name_values = {
+            None: 1,
+            "": 1,
+            "_squared": 2,
+            "_cubed": 3,
+            "_to_the_fourth": 4,
+            "_to_the_fifth": 5,
+            # that's the highest I've ever seen
+        }
+        pattern = re.compile(r"(per_)?([a-z]+)(_squared|_cubed|_to_the_fourth|_to_the_fifth)?")
+        pairs = []
+        for match in re.finditer(pattern, spec):
+            name = match.group(2)
+            if name not in _EXTANT_UNITS:
+                name = name + "s"
+                if name not in _EXTANT_UNITS:
+                    raise KeyError(f"The requested unit {name} does not exist")
+            unit = _EXTANT_UNITS[name]
+            power = name_values[match.group(3)]
+            if match.group(1):
+                power *= -1
+            pairs.append((unit, power))
+        return cls(tuple(pairs))
+
 
 class UnitInterface:
     """A dummy class for type checking purposes, defining the interface of a unit class."""
@@ -567,3 +670,6 @@ class UnitInterface:
 
     def sig_figs(self, figures=3):
         ...
+
+    def to_latex(self, siunitx_major_version=SIUNITX_NEW):
+        ...
